use strictures 1;
use Test::More;

use DBIx::Data::Store::CRUD;
use DBIx::Data::Store::Raw;

use DBI;
use Scalar::Util qw(refaddr);

sub sort_set {
  sort { $a->{name} cmp $b->{name} } @_
}

my $dsn = 'dbi:SQLite:tmp.db';

sub setup_dbh {
  unlink('tmp.db');
  return DBI->connect($dsn, undef, undef, { RaiseError => 1 })
}

sub setup_db {
  my $dbh = setup_dbh;
  $dbh->do(q{
    CREATE TABLE names (
      id INTEGER NOT NULL PRIMARY KEY,
      name VARCHAR(255) NOT NULL
    )
  });
  #my $pop = $dbh->prepare(q{INSERT INTO person (name) VALUES (?)});
  #my @names = qw(Joe Jim Bob Pterry);
  #$pop->execute($_) for @names;
  #return sort_set do {
  #  my $id = 0; map +{ id => ++$id, name => $_ }, @names
  #};
}

my $db_store = DBIx::Data::Store::Raw->connect($dsn);

sub raw_store { $db_store }

sub make_store {
  my ($crud) = @_;
  DBIx::Data::Store::CRUD->new({
    raw => $db_store,
    sql => {
      select_all => 'SELECT id, name FROM names',
      delete_all => 'DELETE FROM names',
      select_one => 'SELECT id, name FROM names WHERE id = ?',
      insert_one => sub {
        my ($store, undef, $dbh, $args) = @_;
        $store->_sth_for($dbh, 'INSERT INTO names (name) VALUES (?)', $args);
        [ $dbh->last_insert_id(undef,undef,undef,undef), @$args ];
      },
      update_one => 'UPDATE names SET name = ? WHERE id = ?',
      delete_one => 'DELETE FROM names WHERE id = ?',
    },
  });
}

sub run_tests {
  setup_db;

  my $store = make_store;

  is_deeply([$store->flatten], [], 'Empty set');

  is_deeply($store->add(['Bob']), [1,'Bob'], 'Add record');

  is_deeply([$store->flatten], [[1,'Bob']], 'One member');

  is_deeply($store->get([1]), [1,'Bob'], 'Retrieve by key');

  $store->replace([1],['Robert']);

  is_deeply([$store->flatten], [[1,'Robert']], 'Name changed (all)');

  is_deeply($store->get([1]), [1,'Robert'], 'Retrieve by key');

  $store->add([$_]) for qw(Joe James Jim);

  my $flatsort = sub {
    [ sort { $a->[0] <=> $b->[0] } $store->flatten ]
  };

  is_deeply($flatsort->(), [
    [1,'Robert'],[2,'Joe'],[3,'James'],[4,'Jim']
  ], 'Four members');

  is_deeply($store->get([3]),[3,'James'], 'Retrieve by key');

  $store->remove([3]);

  is_deeply($flatsort->(), [
    [1,'Robert'],[2,'Joe'],[4,'Jim']
  ], 'Three members left');

  done_testing;
}

run_tests unless caller;
